import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, Component, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import * as CodeMirror from 'codemirror';
import { CommonModule } from '@angular/common';

/**
 * CodeMirror component
 * Usage :
 * <lt-codemirror [(ngModel)]="data" [config]="{...}"></lt-codemirror>
 */
class LtCodemirrorComponent {
    /**
     * Constructor
     */
    constructor() {
        // tslint:disable-next-line:no-output-native
        this.change = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.focus = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.blur = new EventEmitter();
        this.cursorActivity = new EventEmitter();
        this.val = '';
    }
    get value() { return this.val; }
    set value(v) {
        if (v !== this.val) {
            this.val = v;
            this.onChange(v);
        }
    }
    /**
     * On component destroy
     */
    ngOnDestroy() { }
    /**
     * On component view init
     */
    ngAfterViewInit() {
        this.config = this.config || {};
        this.codemirrorInit(this.config, this.size);
    }
    /**
     * Initialize codemirror
     */
    codemirrorInit(config, size) {
        this.instance = CodeMirror.fromTextArea(this.host.nativeElement, config);
        this.instance.setValue(this.val);
        this.instance.on('change', () => {
            this.updateValue(this.instance.getValue());
        });
        this.instance.on('focus', (instance, event) => {
            this.focus.emit({ instance, event });
        });
        this.instance.on('cursorActivity', (instance) => {
            this.cursorActivity.emit({ instance });
        });
        this.instance.on('blur', (instance, event) => {
            this.blur.emit({ instance, event });
        });
        if (this.size !== undefined) {
            this.instance.setSize(this.size.w, this.size.h);
        }
    }
    /**
     * Value update process
     */
    updateValue(value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    }
    /**
     * Implements ControlValueAccessor
     */
    writeValue(value) {
        this.val = value || '';
        if (this.instance) {
            this.instance.setValue(this.val);
        }
    }
    onChange(v) { }
    onTouched() { }
    registerOnChange(fn) { this.onChange = fn; }
    registerOnTouched(fn) { this.onTouched = fn; }
}
LtCodemirrorComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
LtCodemirrorComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.2.16", type: LtCodemirrorComponent, selector: "lt-codemirror", inputs: { config: "config", size: "size", value: "value" }, outputs: { change: "change", focus: "focus", blur: "blur", cursorActivity: "cursorActivity", instance: "instance" }, providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => LtCodemirrorComponent),
            multi: true
        }
    ], viewQueries: [{ propertyName: "host", first: true, predicate: ["host"], descendants: true }], ngImport: i0, template: `
    <textarea #host></textarea>
`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorComponent, decorators: [{
            type: Component,
            args: [{
                    // tslint:disable-next-line:component-selector
                    selector: 'lt-codemirror',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => LtCodemirrorComponent),
                            multi: true
                        }
                    ],
                    template: `
    <textarea #host></textarea>
`,
                    styles: []
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { config: [{
                type: Input
            }], size: [{
                type: Input
            }], change: [{
                type: Output
            }], focus: [{
                type: Output
            }], blur: [{
                type: Output
            }], cursorActivity: [{
                type: Output
            }], host: [{
                type: ViewChild,
                args: ['host', { static: false }]
            }], instance: [{
                type: Output
            }], value: [{
                type: Input
            }] } });

class LtCodemirrorModule {
}
LtCodemirrorModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
LtCodemirrorModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorModule, declarations: [LtCodemirrorComponent], imports: [CommonModule], exports: [LtCodemirrorComponent] });
LtCodemirrorModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorModule, imports: [[
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: LtCodemirrorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [LtCodemirrorComponent],
                    exports: [LtCodemirrorComponent]
                }]
        }] });

/*
 * Public API Surface of lt-codemirror
 */

/**
 * Generated bundle index. Do not edit.
 */

export { LtCodemirrorComponent, LtCodemirrorModule };
//# sourceMappingURL=lt-codemirror.js.map
